(* cw: a Morse code program
   Copyright (C) 2005  Eric C. Cooper <ecc@cmu.edu>
   Released under the GNU General Public License *)

type state =
  { dit : Audio.t;
    dah : Audio.t;
    element_pause : Audio.t;
    letter_pause : Audio.t;
    word_pause : Audio.t;
    send : Audio.t -> unit;
    display : char -> unit }

(* Return unit time in milliseconds corresponding to n words per minute.
   One word is taken to be 50 units *)

let wpm n = 1200 / n

let init ~frequency ~fast_wpm ~slow_wpm ~send ~display =
  let fast = wpm fast_wpm and slow = wpm slow_wpm in
  { (* 1 dit = 1 unit *)
    dit = Audio.tone ~hz: frequency ~ms: fast;
    (* 1 dah = 3 dits *)
    dah = Audio.tone ~hz: frequency ~ms: (3*fast);
    (* 1 (fast) unit between dits and dahs *)
    element_pause = Audio.silence fast;
    (* 3 (slow) units between letters, less trailing element pause *)
    letter_pause = Audio.silence (3*slow - fast);
    (* 7 (slow) units between words, less trailing letter pause *)
    word_pause = Audio.silence (4*slow);
    send = send;
    display = display }

let dit p =
  p.send p.dit;
  p.send p.element_pause

let dah p =
  p.send p.dah;
  p.send p.element_pause

let space p =
  p.send p.word_pause

let morse_code char =
  match char with
  | 'A' | 'a' -> [dit;dah]
  | 'B' | 'b' -> [dah;dit;dit;dit]
  | 'C' | 'c' -> [dah;dit;dah;dit]
  | 'D' | 'd' -> [dah;dit;dit]
  | 'E' | 'e' -> [dit]
  | 'F' | 'f' -> [dit;dit;dah;dit]
  | 'G' | 'g' -> [dah;dah;dit]
  | 'H' | 'h' -> [dit;dit;dit;dit]
  | 'I' | 'i' -> [dit;dit]
  | 'J' | 'j' -> [dit;dah;dah;dah]
  | 'K' | 'k' -> [dah;dit;dah]
  | 'L' | 'l' -> [dit;dah;dit;dit]
  | 'M' | 'm' -> [dah;dah]
  | 'N' | 'n' -> [dah;dit]
  | 'O' | 'o' -> [dah;dah;dah]
  | 'P' | 'p' -> [dit;dah;dah;dit]
  | 'Q' | 'q' -> [dah;dah;dit;dah]
  | 'R' | 'r' -> [dit;dah;dit]
  | 'S' | 's' -> [dit;dit;dit]
  | 'T' | 't' -> [dah]
  | 'U' | 'u' -> [dit;dit;dah]
  | 'V' | 'v' -> [dit;dit;dit;dah]
  | 'W' | 'w' -> [dit;dah;dah]
  | 'X' | 'x' -> [dah;dit;dit;dah]
  | 'Y' | 'y' -> [dah;dit;dah;dah]
  | 'Z' | 'z' -> [dah;dah;dit;dit]

  | '1' -> [dit;dah;dah;dah;dah]
  | '2' -> [dit;dit;dah;dah;dah]
  | '3' -> [dit;dit;dit;dah;dah]
  | '4' -> [dit;dit;dit;dit;dah]
  | '5' -> [dit;dit;dit;dit;dit]
  | '6' -> [dah;dit;dit;dit;dit]
  | '7' -> [dah;dah;dit;dit;dit]
  | '8' -> [dah;dah;dah;dit;dit]
  | '9' -> [dah;dah;dah;dah;dit]
  | '0' -> [dah;dah;dah;dah;dah]

  | '.' -> [dit;dah;dit;dah;dit;dah]
  | ',' -> [dah;dah;dit;dit;dah;dah]
  | '?' -> [dit;dit;dah;dah;dit;dit]
  | '-' -> [dah;dit;dit;dit;dit;dah]
  | '=' -> [dah;dit;dit;dit;dah]
  | ':' -> [dah;dah;dah;dit;dit;dit]
  | ';' -> [dah;dit;dah;dit;dah;dit]
  | '(' -> [dah;dit;dah;dah;dit]
  | ')' -> [dah;dit;dah;dah;dit;dah]
  | '/' -> [dah;dit;dit;dah;dit]
  | '"' -> [dit;dah;dit;dit;dah;dit]
  | '$' -> [dit;dit;dit;dah;dit;dit;dah]
  | '\'' -> [dit;dah;dah;dah;dah;dit]
  | '_' -> [dit;dit;dah;dah;dit;dah]
  | '+' -> [dit;dah;dit;dah;dit]
  | '@' -> [dit;dah;dah;dit;dah;dit]

  | ' ' | '\n' -> [space]

  | _ -> failwith ("no Morse code for " ^ Char.escaped char)

let characters =
  "ABCDEFGHIJKLMNOPQRSTUVWXYZ\
   abcdefghijklmnopqrstuvwxyz\
   1234567890\
   .,?-=:;()/\"$'_+@"

let valid_char = String.contains characters

let send_letter p c =
  p.display c;
  List.iter (fun proc -> proc p) (morse_code c);
  p.send p.letter_pause

let send p str =
  for i = 0 to String.length str - 1 do
    send_letter p str.[i]
  done
